setClass("RPPA",
         representation = list(
           data="data.frame",
           file="character"
           ))

RPPA <- function(filename, path='.') {
  temp <- read.table(file.path(path, filename), fill=TRUE,
                     header=TRUE, row.names=NULL, sep='\t',
                     quote='', comment.char='', skip=4)
  temp <- temp[,1:(ncol(temp)-1)]
  newNames <- dimnames(temp)[[2]]
  newNames <- sub("GeneID",   "Sample",  newNames)
  newNames <- sub("mean\_",   "Mean.",   newNames)
  newNames <- sub("vol\_",    "Vol.",    newNames)
  newNames <- sub("median\_", "Median.", newNames)
  newNames <- sub("net",      "Net",     newNames)
  newNames <- sub("total",    "Total",   newNames)
  newNames <- sub("bkg",      "Bkg",     newNames)
  newNames <- sub("dust",     "Dust",    newNames)
  dimnames(temp)[[2]] <- newNames
  new("RPPA", data=temp, file = filename)
}

setMethod("summary", "RPPA", function(object, ...) {
  cat(paste("An RPPA object loaded from", object@file, "\n\n"))
  summary(object@data)
})

setMethod("image", "RPPA", function(x, measure="Mean.Net", main=measure,
                                    colorbar=FALSE,
                                    col=terrain.colors(256),
                                    zlim=NULL,
                                    ...) {
  temp <- x@data
  my <- max(temp$Main.Row) * max(temp$Sub.Row)
  mx <- max(temp$Main.Col) * max(temp$Sub.Col)
  yspot <- 1+my-(max(temp$Sub.Row)*(temp$Main.Row-1) + temp$Sub.Row)
  xspot <- max(temp$Sub.Col)*(temp$Main.Col-1) + temp$Sub.Col
  geo <- tapply(temp[,measure], list(xspot, yspot), mean)
  if(is.null(zlim)) zlim <- range(geo[is.finite(geo)])
  if (colorbar) {
    ## This block encountered some problems when the
    ## plot region was too small. 
    ##
    ##layout(matrix(c(2, 1), nrow=1), widths=c(10,1))
    ##opar <- par(mai=c(1, 0.5, 1, 0.1))
    ##on.exit(par(opar))
    ##on.exit(layout(1))
    ##image(1, seq(min(geo, na.rm=TRUE), max(geo, na.rm=TRUE), length=256),
    ##      matrix(1:256, nrow=1), col=col,
    ##      xaxt='n', xlab='', ylab='')

    ## first, get the size of the plotting region
    ## in relative units
    startPlt <- par()$plt;
    
    ## We're only going to partition things on the
    ## x-axis, so only the first 2 coordinates are of
    ## interest. Define the boundaries for the two panels
    ## so that a 10/1 width ratio is attained. 
    imagePlt    <- startPlt;
    colorbarPlt <- startPlt;
    startWidth  <- startPlt[2] - startPlt[1];

    imagePlt[2]    <- startPlt[1] + (10/12)*startWidth;
    colorbarPlt[1] <- startPlt[2] - ( 1/12)*startWidth;

    ## draw the colorbar
    par(plt=colorbarPlt);
    image(1, seq(min(zlim, na.rm=TRUE),
                 max(zlim, na.rm=TRUE), length=256),
          matrix(1:256, nrow=1), col=col,
          xaxt='n', yaxt='n', xlab='', ylab='')
    axis(4); ## put labeling at right

    ## set things up to draw the main image and
    ## revert back for the next figure
    par(plt=imagePlt, new=TRUE);
    on.exit(par(plt=startPlt));
  }
  image(1:mx, 1:my, geo, main=main, col=col, zlim=zlim,...)
  abline(h = 0.5 + seq(0, my, length=1+max(temp$Main.Row)))
  abline(v = 0.5 + seq(0, mx, length=1+max(temp$Main.Col)))
  invisible(x)
})



if (FALSE) {
  path <- "../inst/rppaTumorData"
  erk2 <- RPPA("ERK2.txt", path=path)
  summary(erk2)
  image(erk2)
  image(erk2, colorbar=TRUE)
  image(erk2, "Vol.Bkg", colorbar=TRUE)
  rm(path, erk2)
}

